package playground;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderConfiguration;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.definition.KnowledgePackage;
import org.drools.impl.EnvironmentFactory;
import org.drools.io.ResourceFactory;
import org.drools.runtime.Environment;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;
import org.drools.time.SessionPseudoClock;
import org.junit.Assert;
import org.junit.Test;
import playground.datamodel.EventObject;

import java.util.Collection;
import java.util.concurrent.TimeUnit;

/**
 * Date: 5/28/12
 * Time: 1:20 PM
 */
public class AverageTest {
    protected KnowledgeBase kb;
    protected StatefulKnowledgeSession s;
    protected SessionPseudoClock clock;
    protected WorkingMemoryEntryPoint eventStream;

    @Test
    public void testLatestEventUnderAverage() {
        KnowledgeBuilderConfiguration bConf = KnowledgeBuilderFactory.newKnowledgeBuilderConfiguration();
        KnowledgeBuilder builder = KnowledgeBuilderFactory.newKnowledgeBuilder(bConf);
        builder.add(ResourceFactory.newClassPathResource("average.drl"), ResourceType.DRL);
        Assert.assertFalse(builder.getErrors().toString(), builder.hasErrors());
        KnowledgeBaseConfiguration kbConf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        kbConf.setOption(EventProcessingOption.STREAM);
        Collection<KnowledgePackage> packages = builder.getKnowledgePackages();
        kb = KnowledgeBaseFactory.newKnowledgeBase(kbConf);
        kb.addKnowledgePackages(packages);
        KnowledgeSessionConfiguration sConf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
        sConf.setOption(ClockTypeOption.get("pseudo"));
        Environment env = EnvironmentFactory.newEnvironment();
        s = kb.newStatefulKnowledgeSession(sConf, env);
        eventStream = s.getWorkingMemoryEntryPoint("Event Stream");
        clock = (SessionPseudoClock) s.getSessionClock();
        clock.advanceTime(1L, TimeUnit.DAYS);

        Assert.assertTrue(s.fireAllRules() == 0);

        EventObject valid1 = new EventObject(100L,"Full", "success");
        eventStream.insert(valid1);
        clock.advanceTime(1L, TimeUnit.MILLISECONDS);

        Assert.assertTrue(s.fireAllRules() == 0);

        EventObject invalid = new EventObject(0L,"Whatever", "fail");
        eventStream.insert(invalid);
        clock.advanceTime(1L, TimeUnit.MILLISECONDS);

        Assert.assertTrue(s.fireAllRules() == 0);

        EventObject valid2 = new EventObject(99L,"Full", "success");
        eventStream.insert(valid2);
        clock.advanceTime(1L, TimeUnit.MILLISECONDS);

        Assert.assertTrue(valid2.getSize()+" should be under the average of previous matching events",s.fireAllRules() == 1);
    }
}
